declare("Chainfire_tab", {
	version = "1.0.0",
})
local function loadluafile(...)
	local f, err = loadfile((...))
	assert(f and type(f) == "function", err)
	return f(select(2, ...))
end

local T = {
	Tab = {
		title = "Chainfire",
		columns = {
			{
				title = "",
				width = 0,
			},
			{
				title = "--- Chainfire Configuration ---",
				width = 0.7,
			},
			{
				title = "",
				width = 0.3,
			},
		},
		filters = {
			{
				title = "Chain 1",
			},
			{
				title = "Config",
				pages = {
					{
						title = "Config 1",
					},
					{
						title = "Config 2",
					},
				},
			},
		},
		--------------
		delay = {0, 0, 0},
		weapon_groups = {{}, {}, {}},
		config_delay = {"Delay", "+0.1s", "-0.1s", "+0.01s", "-0.01s", "", "Reset Delay"},
		triggers = {"Primary", "Secondary", "Tertiary"},
		trigger_index = -1,
		filter_index = -1,
		page_index = 1,
		saved, -- 1 == success, 2 == fail
		written, -- 1 == success, 2 == fail
	}
}
--Chainfire_tab.T = T

local function check_triggers(groups, newID)
	if     (newID == groups[1]) then table.remove(groups, 1)
	elseif (newID == groups[2]) then table.remove(groups, 2)
	elseif (#groups < 2) then
		table.insert(groups, newID)
		table.sort(groups)
	end
end

local function save_chain(self)
	-- Saves all three triggers with their current values
	if (self.trigger_index < 1) then
		self.saved = 2
		return
	end

	local config = {self.delay, self.weapon_groups}
	gkini.WriteString("Chainfire Tab", "Chain1", spickle(config))

	self.saved = 1
end

local function write_chain(self)
	if (self.trigger_index < 1) then
		self.written = 2
		return
	end

	local groups = self.weapon_groups[self.trigger_index]
	if (not (groups[1] and groups [2])) then
		self.written = 2
		return
	end

	local success, err = pcall(self.BuildAliases, self)
	self.written = success and 1 or 2
end

function T.Tab:ChangeTarget(oldID, newID)
	if (self.filter_index <= 1) then
		if (newID >= 1 and newID <= 3) then
			self.trigger_index = newID -- - 1
			self.saved = 0
			self.written = 0

		elseif (newID == 8) then
			save_chain(self)

		elseif (newID == 9) then
			write_chain(self)
		end

	elseif (self.filter_index == 2) then
		if (self.page_index == 1 and newID > 1) then
			local groups = self.weapon_groups[self.trigger_index]
			if (groups) then
				check_triggers(groups, newID - 1)
			end

		elseif (self.page_index == 2) then
			if (newID >= 3 and newID <= 6) then
				local inc = tonumber(self.config_delay[newID - 1]:sub(1, -2))
				local delay = self.delay[self.trigger_index]
				delay = delay + inc
				if (delay < 0) then delay = 0 end
				self.delay[self.trigger_index] = delay

			elseif (newID == 8) then
				self.delay[self.trigger_index] = 0
			end
		end
	end

	targetlist.API.UpdateDisplay(self, self:BuildList())
end

function T.Tab:ChangeFilter(index, page)
	self.filter_index = index
	self.page_index = page
	self.saved = 0
	self.written = 0
	targetlist.API.UpdateDisplay(self, self:BuildList())
end

function T.Tab:BuildList()

	local list = {}
	if (self.filter_index <= 1) then
		--list[1] = {"H", "Chainfire 1"} -- May be used later to display chainfire config names, if multiple are supported

		for i, trigger in ipairs(self.triggers) do
			list[i] = {i, trigger.." Trigger", highlight = i == self.trigger_index}
		end
		table.insert(list, {})
		local groups = self.weapon_groups[self.trigger_index]
		table.insert(list, {"G", "Weapon Groups: "..(groups and ((groups[1] or "?").." - "..(groups[2] or "?"))	or "? - ?")})
		table.insert(list, {"D", "Delay: "..(self.delay[self.trigger_index] or "?")})
		table.insert(list, {})
		table.insert(list, {"S", (self.saved == 1 and "\12700FF00" or self.saved == 2 and "\127FF0000" or "").."Save"})
		table.insert(list, {"W", (self.written == 1 and "\12700FF00" or self.written == 2 and "\127FF0000" or "").."Write"})

	elseif (self.filter_index == 2) then

		if (self.trigger_index < 1) then
			list[1] = {"", "Select a trigger to configure."}

		elseif (self.page_index == 1) then
			list[1] = {"H", "\127FF0000"..self.triggers[self.trigger_index].." Trigger"}
			for i = 1, 6 do
				local groups = self.weapon_groups[self.trigger_index]
				list[i + 1] = {
					i,
					"Weapon Group "..i,
					groups and (groups[1] == i or groups[2] == i) and "\12700FF00@" or nil,
				}
			end
			table.insert(list, {})
			table.insert(list, {"M", "Select the first and last group."})

		elseif (self.page_index == 2) then
			list[1] = {"H", "\127FF0000"..self.triggers[self.trigger_index].." Trigger"}
			for i, s in ipairs(self.config_delay) do
				local j = i + 1
				list[j] = {
					j,
					j == 2 and "\12700FF00"..s or s,
					j == 2 and "\12700FF00"..self.delay[self.trigger_index] or nil,
					--highlight = j == 2,
				}
			end
		end
	end
	return list
end


function T.Tab:BuildAliases()
	--[[
	CHainfire Module from Toastercrush Plugins Suite, written by Scuba Steve 9.0. See README for installation instructions and descriptions.

	Copyright info:
	Copyright Tyler Gibbons(Scuba Steve 9.0), 2008-2011. Feel free to use, distribute, and modify any and all sections of code as
		long as I am attributed as the original author of any code used, modified, or distributed. If you make something cool
		using this code, I'd love to know. Send me a message on IRC @ slashnet.org as 'kavec' and say hi!
	--]]

	local gkpc = gkinterface.GKProcessCommand
	local trigger = self.trigger_index
	local fgroup = self.weapon_groups[trigger][1]
	local lgroup = self.weapon_groups[trigger][2]
	local delay = self.delay[trigger]
	local next = 0
	local i = 0
	local damnVO = 0

	--Horsies!
	-- Thanks to VO being weird, damnVO is to correct for +shoot1 being secondary trigger and +shoot2 being primary trigger. Gar
	if(trigger == 1) then
		trigger = 1
		damnVO = 2
	elseif(trigger == 2) then
		trigger = 2
		damnVO = 1
	else
		trigger = 3
		damnVO = 3
	end

	--First, the gruntwork alias state machine.
	--Each set has its own state machine to run off of so things don't get tangled. That would be bad. We don't like tangles.
	i = fgroup
	gkpc("alias cfireswitch"..trigger.." \"cfiregrunt"..i.."_"..trigger.."\"")
	while(i <= lgroup) do
		next = i + 1
		if(i ~= lgroup) then
			gkpc("alias cfiregrunt"..i.."_"..trigger.." \"Weapon"..i.."; alias cfireswitch"..trigger.." cfiregrunt"..next.."_"..trigger.."\"")
		else
			gkpc("alias cfiregrunt"..i.."_"..trigger.." \"Weapon"..i.."; alias cfireswitch"..trigger.." cfiregrunt"..fgroup.."_"..trigger.."\"")
		end
		i = i + 1
	end

	--Now on to control structures

	gkpc("alias +cfire"..trigger.." \"+Shoot"..damnVO.."; cfiremakeloop"..trigger.."; cfirelstart"..trigger.."\"")
	gkpc("alias cfirelstart"..trigger.." \"cfireloop"..trigger.."; alias cfirelstart"..trigger.." none\"")
	gkpc("alias cfireloop"..trigger.." none")
	gkpc("alias cfiremakeloop"..trigger.." \"alias cfireloop"..trigger.." \'cfireswitch"..trigger.."; wait "..delay.." cfireloop"..trigger.."\'\"")
	gkpc("alias -cfire"..trigger.." \"+Shoot"..damnVO.." 0 ; alias cfireloop"..trigger.." none; alias cfirelstart"..trigger.." \'cfireloop"..trigger.."; alias cfirelstart"..trigger.." none\'\"")
	print("Now bind +cfire"..trigger.." to something and it should work. Made with "..delay.."s delay.")
end

function T.Tab:TARGETLIST_TAB_CHANGED()
	self.is_active = targetlist.API.IsActiveTab(T.Tab)
	if (self.is_active) then
		targetlist.API.UpdateDisplay(self, self:BuildList())
	end
end

function T.Tab:PLUGINS_LOADED()
	if (isdeclared("targetlist") and targetlist.API) then
		if (targetlist.API.Version >= 2) then
			targetlist.API.RegisterTab(T.Tab)
			console_print("Chainfire Tab for MDI-TargetList loaded (v"..Chainfire_tab.version..").")
		else
			print("\127FFFFFFThe MDI-TargetList API version is different than expected.  Chainfire Tab was not loaded.\127o")
		end
	end
end

function T.Tab:PLAYER_ENTERED_GAME()
	if (not (isdeclared("targetlist") and targetlist.API)) then
		print("\127FFFFFFThe Chainfire Tab requires MDI-TargetList v4.1+.\127o")
	else
		local config = gkini.ReadString("Chainfire Tab", "Chain1", "")
		config = unspickle(config)
		self.delay = config[1] or {0, 0, 0}
		self.weapon_groups = config[2] or {{}, {}, {}}
	end
end

RegisterEvent(T.Tab, "TARGETLIST_TAB_CHANGED")
RegisterEvent(T.Tab, "PLUGINS_LOADED")
RegisterEvent(T.Tab, "PLAYER_ENTERED_GAME")
